
module tmp/simpleUMLCD
open util/boolean as Bool

// Meta-model structure //
sig ClassModel {
	classifier: set Classifier,
	association: set Association
}

abstract sig Classifier {
      name :  Int
	
	}

sig PrimitiveDataType extends Classifier {}

sig Class extends Classifier {
	is_persistent: one Bool,
	parent : lone Class,
	attrs : some Attribute //atleast one attribute, chnqge to set if not necessary
}

sig Association {
	name: Int,
	dest: one Class,
	src: one Class
}


sig Attribute {
	name: Int,
	is_primary : Bool,
	type: one Classifier
}


//Meta-model constraints//

//i
//There must be no cyclic inheritance in the generated UML class diagram
fact noCyclicInheritance {
	no c: Class | c in c.^parent
}

//ii
//All the attributes in a Class must have unique attribute names
fact uniqueAttributeNames {
	all c:Class |  all a1:  c.attrs , a2 : c.attrs |a1.name==a2.name => a1 = a2  
}

//iii
//An attribute object can be contained by only one class
fact attributeContainment {
	all c1:Class, c2:Class | all a1:c1.attrs, a2:c2.attrs | a1==a2 => c1=c2
}

//iv
//There is exactly one ClassModel object
fact oneClassModel {
	one ClassModel
}

//v
//All Classifier objects are contained in a ClassModel
fact classifierContainment {
	all c:Classifier | c in ClassModel.classifier
}

//vi
//All Association objects are contained in a ClassModel
fact associationContainment {
	all a:Association | a in ClassModel.association
}

//vii
//All attributes in model
fact allAttribInModels
{
	all a:Attribute | some c: Class | a in c.attrs
}

//viii
//A Classifier must have a unique name in the class diagram
fact uniqueClassifierName {
	all c1:Classifier, c2:Classifier | c1.name==c2.name => c1=c2
}

//ix
//An associations have the same name either they are the same association or they have different sources
fact uniqeNameAssocSrc {
	all a1:Association, a2:Association | a1.name == a2.name => (a1 = a2 or a1.src != a2.src)
}

//x
//A Class cannot have an association and an attribute of the same name 
fact noAttribAndAssocSameName{
	all c:Class, assoc:Association | all a : c.attrs | (assoc.src == c) => a.name != assoc.name
}

// Initial pre-conditions //

//Model Transformation UMLCD to RDBMS Pre-condition
fact atleastOnePrimaryAttribute {
    all c:Class| some a:c.attrs | a.is_primary==True
}

// Navigating the type, we need to find primitive types (at worst at the level 4)
fact no4CyclicClassAttribute{
	all a:Attribute | a.type in Class => all a1:a.type.attrs|a1.type in Class => all a2:a.type.attrs|a2.type in Class => all a3:a.type.attrs|a3.type in Class => all a4:a.type.attrs|a4.type in PrimitiveDataType
}

// No association between two classes of an inheritance tree
fact noAssocBetweenClassInHierarchy
{
	all a : Association | all c: Class | (a.src =c implies a.dest not in c.^parent) and (a.dest =c implies a.src not in c.^parent)
}



//Model Fragments

pred partition1()
{
 some c:Classifier|c.name=0
}

pred partition2()
{
 some c:Classifier|c.name>=0
}

pred partition3()
{
some c:Class |c.is_persistent=True
}

pred partition4()
{
some c:Class |c.is_persistent=False
}

pred partition5()
{
some c:Class|#c.parent=0 
}

pred partition6()
{
some c:Class|#c.parent=1 
}

pred partition7()
{
some c:Class|#c.attrs=0 
}

pred partition8()
{
some c:Class|#c.attrs=1
}

pred partition9()
{
some c:Class|#c.attrs>1
}

pred partition10()
{
some a:Attribute|a.is_primary=True
}

pred partition11()
{
some a:Attribute|a.is_primary=False
}

pred partition12()
{
some a:Attribute|a.name=0
}

pred partition13()
{
some a:Attribute|a.name!=0
}

pred partition14()
{
some a:Attribute|#a.type=1
}

pred partition15()
{
some a:Association|a.name=0 
}

pred partition16()
{
some a:Association|a.name!=0 
}


pred partition17()
{
some a:Association|#a.dest=1 
}

pred partition18()
{
some a:Association|#a.src=1 
}

pred partition19()
{
some c:ClassModel|#c.classifier=0
}

pred partition20()
{
some c:ClassModel|#c.classifier=1
}

pred partition21()
{
some c:ClassModel|#c.classifier>1
}

pred partition22()
{
some c:ClassModel|#c.association=0
}


pred partition23()
{
some c:ClassModel|#c.association=1
}


pred partition24()
{
some c:ClassModel|#c.association>1
}

//Just the meta-model

pred noFragments()

{

}
//Model Fragments from AllRanges


pred mfAllRanges1()
{
partition1
}

pred mfAllRanges2()
{
partition2
}
pred mfAllRanges3()
{
partition3
}
pred mfAllRanges4()
{
partition4
}

pred mfAllRanges5()
{
partition5
}



pred mfAllRanges6()
{
partition6
}

pred mfAllRanges7()
{
partition7
}
pred mfAllRanges8()
{
partition8
}
pred mfAllRanges9()
{
partition9
}
pred mfAllRanges10()
{
partition10
}
pred mfAllRanges11()
{
partition11
}

pred mfAllRanges12()
{
partition12
}
pred mfAllRanges13()
{
partition13
}

pred mfAllRanges14()
{
partition14
}

pred mfAllRanges15()
{
partition15
}

pred mfAllRanges16()
{
partition16
}


pred mfAllRanges17()
{
partition17
}


pred mfAllRanges18()
{
partition18
}

pred mfAllRanges19()
{
partition19
}

pred mfAllRanges20()
{
partition20
}

pred mfAllRanges21()
{
partition21
}

pred mfAllRanges22()
{
partition22
}

pred mfAllRanges23()
{
partition23
}
pred mfAllRanges24()
{
partition24
}


//Model Fragments from AllPartitions
pred mfAllPartitions1()
{
partition1 and partition2
}

pred mfAllPartitions2()
{
partition3 and partition4
}


pred mfAllPartitions3()
{
partition5 and partition6
}


pred mfAllPartitions4()
{
partition7 and partition8 and partition9
}


pred mfAllPartitions5()
{
partition10 and partition11 
}


pred mfAllPartitions6()
{
partition12 and partition13
}

pred mfAllPartitions7()
{
partition14
}

pred mfAllPartitions8()
{
partition15 and partition16
}


pred mfAllPartitions9()
{
partition17
}

pred mfAllPartitions10()
{
partition18
}


pred mfAllPartitions11()
{
partition19 and partition20 and partition21
}

pred mfAllPartitions12()
{
partition22 and partition23 and partition24
}


//All Partitions Weeded


pred mfAllPartitionsWeeded1()
{
//partition7 and //Weeded
partition8 and partition9
}


pred mfAllPartitionsWeeded2()
{
partition24 
//partition22 and partition23 //Weeded
}






//check checkassocPersistentClass for 20
 //Synthesized run statements for experiments ..

//All Partitions

//[Class,Association,Attribute,PrimitiveDataType]
//Set 1 [5,5,25,4]

run mfAllPartitions1 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association

/*
//[Class,Association,Attribute,PrimitiveDataType]
// Set 2 [5,10,25,4]

//All Partitions
run mfAllPartitions1 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 5 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association


//[Class,Association,Attribute,PrimitiveDataType]
//Set 3 [10,5,25,4]



//All Partitions

run mfAllPartitions1 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association

//[Class,Association,Attribute,PrimitiveDataType]
//Set 4 [10,10,25,4]


//All Partitions
run mfAllPartitions1 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 10 Class,exactly 25 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association

//[Class,Association,Attribute,PrimitiveDataType]
//Set 5[5,5,30,4]


//All Partitions
run mfAllPartitions1 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association

//[Class,Association,Attribute,PrimitiveDataType]
//Set 6 [10,5,30,4]


//All Partitions
run mfAllPartitions1 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 10 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 5 Association


//[Class,Association,Attribute,PrimitiveDataType]
//Set  7 [5,10,30,4]


//All Partitions
run mfAllPartitions1 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions2 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions3 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions4 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions5 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions6 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions7 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions8 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions9 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
run mfAllPartitions10 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions11 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association
//run mfAllPartitions12 for 1 ClassModel,5 int,exactly 5 Class,exactly 30 Attribute,exactly 4 PrimitiveDataType,exactly 10 Association

*/